#ifndef _DEFS_H_
#define _DEFS_H_

#define R do{printf("%s:%d: error\n",__FILE__,__LINE__);exit(1);}while(0)

extern int _log;
#define LOG(...) do{if(_log)printf(__VA_ARGS__);}while(0)

#include "queue.h"
#include "options.h"

typedef struct {
  char *sender;
  char *content;
  int content_size;
} message;

typedef struct {
  message *data;
  int size;
  int maxsize;
} messages;

typedef struct {
  char v[32];
} key;

typedef struct chat {
  int update_running;
  int send_running;
  void *data_mutex;
  void *network_mutex;
  int id;
  char *passphrase;
  char *my_name;
  char *her_name;
  char *channel;
  messages m;
  unsigned long iv_high;
  unsigned int iv_low;
  key key1;
  key key2;
  char iv[16];
  void *server_data;
  void (*connect)(struct chat *);
  void (*poll)(struct chat *);
  void (*disconnect)(struct chat *);
  void (*send)(struct chat *, char *);
  void (*send_over)(struct chat *);
  int (*new_message)(struct chat *, const char *);
  void *gui;
  queue send_queue;
  options options;
  int flash_x;
  int flash_y;
  int flash_width;
  int flash_height;
  int flash_count;
  int flash_on;
  int flash_off;
  char *flash_color;
#ifdef FLASH_WITH_GTK
  int flash_running;
  int flash_cur_count;
#endif
} chat;

void set_my_name(chat *, char *);
void set_her_name(chat *, char *);
int extract_messages(chat *, char *);

void s_server_init(chat *);
void mathim_server_init(chat *);

void clear_messages(chat *);
void add_message(chat *, char *, int);

char *decode_message(chat *, char *, int *);
char *encode_message(chat *, char *);

void send_message(chat *, char *);

void crypto_init(chat *);
char *cipher(chat *, char *);
char *decipher(chat *, char *, int *);

void oom();

void acquire_gui(void);
void release_gui(void);

void gui_update_timer_start(void *, int);

#endif /* _DEFS_H_ */
